import os, uuid, sys
from azure.storage.filedatalake import DataLakeServiceClient
from azure.core._match_conditions import MatchConditions
from azure.storage.filedatalake._models import ContentSettings


def initialize_storage_account(storage_account_name, storage_account_key):
    try:  
        global service_client
        service_client = DataLakeServiceClient(account_url="{}://{}.dfs.core.windows.net".format(
            "https", storage_account_name), credential=storage_account_key)
    
    except Exception as e:
        print(e)
        
def list_directory_contents(container, directory):
    try:
        
        file_system_client = service_client.get_file_system_client(file_system=container)

        paths = file_system_client.get_paths(path=directory)

        for path in paths:
            print(path.name)

    except Exception as e:
        print(e)
        
def create_file_system(container):
    try:
        global file_system_client

        file_system_client = service_client.create_file_system(file_system=container)
    
    except Exception as e:
        print(e)
        
        
def create_directory(container, directory):
    try:
        file_system_client = service_client.get_file_system_client(file_system=container)
        file_system_client.create_directory(directory)
    
    except Exception as e:
        print(e)

        
def rename_directory(container, old_directory_name, new_directory_name):
    try:
        file_system_client = service_client.get_file_system_client(file_system=container)
        directory_client = file_system_client.get_directory_client(old_directory_name)

        directory_client.rename_directory(new_name=directory_client.file_system_name + '/' + new_directory_name)
    except Exception as e:
        print(e)
        

def delete_directory(container, directory):
    try:
        file_system_client = service_client.get_file_system_client(file_system=container)
        directory_client = file_system_client.get_directory_client(directory)

        directory_client.delete_directory()
    except Exception as e:
        print(e)
    
    
def upload_file_to_directory(container, directory, online_file_name, local_file_path):
    try:

        file_system_client = service_client.get_file_system_client(file_system=container)

        directory_client = file_system_client.get_directory_client(directory)
        
        file_client = directory_client.create_file(online_file_name)
        local_file = open(local_file_path,'r')

        file_contents = local_file.read()

        file_client.append_data(data=file_contents, offset=0, length=len(file_contents))

        file_client.flush_data(len(file_contents))

    except Exception as e:
        print(e)
        

'''For uploading large files'''
def upload_file_to_directory_bulk(container, directory, online_file_name, local_file_path, overwrite=True):
    try:

        file_system_client = service_client.get_file_system_client(file_system=container)

        directory_client = file_system_client.get_directory_client(directory)
        
        file_client = directory_client.get_file_client(online_file_name)

        local_file = open(local_file_path,'r')

        file_contents = local_file.read()

        file_client.upload_data(file_contents, overwrite=overwrite)

    except Exception as e:
        print(e)


def download_file_from_directory(container, directory, online_file_name, local_file_path):
    try:
        file_system_client = service_client.get_file_system_client(file_system=container)

        directory_client = file_system_client.get_directory_client(directory)
        
        local_file = open(local_file_path,'wb')

        file_client = directory_client.get_file_client(online_file_name)

        download = file_client.download_file()

        downloaded_bytes = download.readall()

        local_file.write(downloaded_bytes)

        local_file.close()

    except Exception as e:
        print(e)


from configparser import ConfigParser
import psycopg2
import psycopg2.extras
from os.path import join, dirname

def config(filename=join(dirname(__file__), 'database.ini'), section='prod-postgresql'):
    # create a parser
    parser = ConfigParser()
    # read config file
    parser.read(filename)

    # get section, default to postgresql
    db = {}
    if parser.has_section(section):
        params = parser.items(section)
        for param in params:
            db[param[0]] = param[1]

        print(f'Trying to connect to {section}')
    else:
        raise Exception(
            'Section {0} not found in the {1} file'.format(section, filename))

    return db



# Use this file for testing the python functions defined elsewhere

"""Test db connection"""


def connect():
    """ Connect to the PostgreSQL database server """
    conn = None
    try:
        # read connection parameters
        params = config()

        # connect to the PostgreSQL server
        print('Connecting to the PostgreSQL database...')
        conn = psycopg2.connect(**params)

        # create a cursor
        cur = conn.cursor()

        # execute a statement
        print('PostgreSQL database version:')
        cur.execute('SELECT version()')

        # display the PostgreSQL database server version
        db_version = cur.fetchone()
        print(db_version)

        # close the communication with the PostgreSQL
        cur.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()
            print('Database connection closed.')


def get_db_conn(section:str = 'prod-postgresql'):
    """ Connect to the PostgreSQL database server """
    conn = None
    try:
        # read connection parameters
        params = config(section=section)

        # connect to the PostgreSQL server
        print('Connecting to the PostgreSQL database...')
        conn = psycopg2.connect(**params)

        print('Connected successfully...')
        return conn

    except (Exception, psycopg2.DatabaseError) as error:
        print(error)


def close_db_conn(conn):
    if conn is not None:
        conn.close()
        print('Database connection closed.')
    else:
        print('Database connection not closed because it doesn\'t exist.')


# # Connect to an existing database
# conn = get_db_conn()

# # Open a cursor to perform database operations
# cur = conn.cursor(cursor_factory=psycopg2.extras.NamedTupleCursor)



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='givvableutils',
    version='0.0.2',    
    description='utility tools for givvable',
    py_modules=['db', 'blob'],
    package_dir={'': 'src'},
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://dev.azure.com/givvable/givvable%20data/_git/givvableutils',
    author='Jatin Wadhwa',
    author_email='jatin@givvable.com',
    license='MIT',
    # packages=['db'],
    install_requires=[
        'psycopg2-binary >= 2.9.3',
        'configparser >= 5.3.0',
        'azure-storage-file-datalake >= 12.8.0',
        'azure-core >= 1.24.0'
    ],
    extras_require={
        "dev": [
            "pytest >= 3.7",
            "twine >= 3.4.2"
        ],
    },

    classifiers=[
        'Development Status :: 1 - Planning',
        # 'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',  
        'Operating System :: OS Independent',        
        # 'Programming Language :: Python :: 2',
        # 'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)